@@ -18,35 +18,20 @@ package org.apache.commons.math3.util;
 
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
 import java.math.BigInteger;
 
 import org.apache.commons.math3.exception.MathArithmeticException;
 import org.apache.commons.math3.exception.MathIllegalArgumentException;
-import org.apache.commons.math3.random.RandomDataImpl;
+import org.apache.commons.math3.random.RandomDataGenerator;
 import org.junit.Assert;
 import org.junit.Test;
 
 /**
  * Test cases for the {@link ArithmeticUtils} class.
  *
- * @version $Id: ArithmeticUtilsTest.java 1244107 2012-02-14 16:17:55Z erans $
  */
 public class ArithmeticUtilsTest {
 
-    /** cached binomial coefficients */
-    private static final List<Map<Integer, Long>> binomialCache = new ArrayList<Map<Integer, Long>>();
-
-    /** Verify that b(0,0) = 1 */
-    @Test
-    public void test0Choose0() {
-        Assert.assertEquals(ArithmeticUtils.binomialCoefficientDouble(0, 0), 1d, 0);
-        Assert.assertEquals(ArithmeticUtils.binomialCoefficientLog(0, 0), 0d, 0);
-        Assert.assertEquals(ArithmeticUtils.binomialCoefficient(0, 0), 1);
-    }
-
     @Test
     public void testAddAndCheck() {
         int big = Integer.MAX_VALUE;
@@ -76,218 +61,13 @@ public class ArithmeticUtilsTest {
         Assert.assertEquals(1, ArithmeticUtils.addAndCheck(2L, -1L));
         Assert.assertEquals(-3, ArithmeticUtils.addAndCheck(-2L, -1L));
         Assert.assertEquals(min, ArithmeticUtils.addAndCheck(min + 1, -1L));
+        Assert.assertEquals(-1, ArithmeticUtils.addAndCheck(min, max));
         testAddAndCheckLongFailure(max, 1L);
         testAddAndCheckLongFailure(min, -1L);
         testAddAndCheckLongFailure(1L, max);
         testAddAndCheckLongFailure(-1L, min);
-    }
-
-
-    @Test
-    public void testBinomialCoefficient() {
-        long[] bcoef5 = {
-            1,
-            5,
-            10,
-            10,
-            5,
-            1 };
-        long[] bcoef6 = {
-            1,
-            6,
-            15,
-            20,
-            15,
-            6,
-            1 };
-        for (int i = 0; i < 6; i++) {
-            Assert.assertEquals("5 choose " + i, bcoef5[i], ArithmeticUtils.binomialCoefficient(5, i));
-        }
-        for (int i = 0; i < 7; i++) {
-            Assert.assertEquals("6 choose " + i, bcoef6[i], ArithmeticUtils.binomialCoefficient(6, i));
-        }
-
-        for (int n = 1; n < 10; n++) {
-            for (int k = 0; k <= n; k++) {
-                Assert.assertEquals(n + " choose " + k, binomialCoefficient(n, k), ArithmeticUtils.binomialCoefficient(n, k));
-                Assert.assertEquals(n + " choose " + k, binomialCoefficient(n, k), ArithmeticUtils.binomialCoefficientDouble(n, k), Double.MIN_VALUE);
-                Assert.assertEquals(n + " choose " + k, FastMath.log(binomialCoefficient(n, k)), ArithmeticUtils.binomialCoefficientLog(n, k), 10E-12);
-            }
-        }
-
-        int[] n = { 34, 66, 100, 1500, 1500 };
-        int[] k = { 17, 33, 10, 1500 - 4, 4 };
-        for (int i = 0; i < n.length; i++) {
-            long expected = binomialCoefficient(n[i], k[i]);
-            Assert.assertEquals(n[i] + " choose " + k[i], expected,
-                ArithmeticUtils.binomialCoefficient(n[i], k[i]));
-            Assert.assertEquals(n[i] + " choose " + k[i], expected,
-                ArithmeticUtils.binomialCoefficientDouble(n[i], k[i]), 0.0);
-            Assert.assertEquals("log(" + n[i] + " choose " + k[i] + ")", FastMath.log(expected),
-                ArithmeticUtils.binomialCoefficientLog(n[i], k[i]), 0.0);
-        }
-    }
-
-    @Test
-    public void testBinomialCoefficientFail() {
-        try {
-            ArithmeticUtils.binomialCoefficient(4, 5);
-            Assert.fail("expecting MathIllegalArgumentException");
-        } catch (MathIllegalArgumentException ex) {
-            // ignored
-        }
-
-        try {
-            ArithmeticUtils.binomialCoefficientDouble(4, 5);
-            Assert.fail("expecting MathIllegalArgumentException");
-        } catch (MathIllegalArgumentException ex) {
-            // ignored
-        }
-
-        try {
-            ArithmeticUtils.binomialCoefficientLog(4, 5);
-            Assert.fail("expecting MathIllegalArgumentException");
-        } catch (MathIllegalArgumentException ex) {
-            // ignored
-        }
-
-        try {
-            ArithmeticUtils.binomialCoefficient(-1, -2);
-            Assert.fail("expecting MathIllegalArgumentException");
-        } catch (MathIllegalArgumentException ex) {
-            // ignored
-        }
-        try {
-            ArithmeticUtils.binomialCoefficientDouble(-1, -2);
-            Assert.fail("expecting MathIllegalArgumentException");
-        } catch (MathIllegalArgumentException ex) {
-            // ignored
-        }
-        try {
-            ArithmeticUtils.binomialCoefficientLog(-1, -2);
-            Assert.fail("expecting MathIllegalArgumentException");
-        } catch (MathIllegalArgumentException ex) {
-            // ignored
-        }
-
-        try {
-            ArithmeticUtils.binomialCoefficient(67, 30);
-            Assert.fail("expecting MathArithmeticException");
-        } catch (MathArithmeticException ex) {
-            // ignored
-        }
-        try {
-            ArithmeticUtils.binomialCoefficient(67, 34);
-            Assert.fail("expecting MathArithmeticException");
-        } catch (MathArithmeticException ex) {
-            // ignored
-        }
-        double x = ArithmeticUtils.binomialCoefficientDouble(1030, 515);
-        Assert.assertTrue("expecting infinite binomial coefficient", Double
-            .isInfinite(x));
-    }
-
-    /**
-     * Tests correctness for large n and sharpness of upper bound in API doc
-     * JIRA: MATH-241
-     */
-    @Test
-    public void testBinomialCoefficientLarge() throws Exception {
-        // This tests all legal and illegal values for n <= 200.
-        for (int n = 0; n <= 200; n++) {
-            for (int k = 0; k <= n; k++) {
-                long ourResult = -1;
-                long exactResult = -1;
-                boolean shouldThrow = false;
-                boolean didThrow = false;
-                try {
-                    ourResult = ArithmeticUtils.binomialCoefficient(n, k);
-                } catch (MathArithmeticException ex) {
-                    didThrow = true;
-                }
-                try {
-                    exactResult = binomialCoefficient(n, k);
-                } catch (MathArithmeticException ex) {
-                    shouldThrow = true;
-                }
-                Assert.assertEquals(n + " choose " + k, exactResult, ourResult);
-                Assert.assertEquals(n + " choose " + k, shouldThrow, didThrow);
-                Assert.assertTrue(n + " choose " + k, (n > 66 || !didThrow));
-
-                if (!shouldThrow && exactResult > 1) {
-                    Assert.assertEquals(n + " choose " + k, 1.,
-                        ArithmeticUtils.binomialCoefficientDouble(n, k) / exactResult, 1e-10);
-                    Assert.assertEquals(n + " choose " + k, 1,
-                        ArithmeticUtils.binomialCoefficientLog(n, k) / FastMath.log(exactResult), 1e-10);
-                }
-            }
-        }
-
-        long ourResult = ArithmeticUtils.binomialCoefficient(300, 3);
-        long exactResult = binomialCoefficient(300, 3);
-        Assert.assertEquals(exactResult, ourResult);
-
-        ourResult = ArithmeticUtils.binomialCoefficient(700, 697);
-        exactResult = binomialCoefficient(700, 697);
-        Assert.assertEquals(exactResult, ourResult);
-
-        // This one should throw
-        try {
-            ArithmeticUtils.binomialCoefficient(700, 300);
-            Assert.fail("Expecting MathArithmeticException");
-        } catch (MathArithmeticException ex) {
-            // Expected
-        }
-
-        int n = 10000;
-        ourResult = ArithmeticUtils.binomialCoefficient(n, 3);
-        exactResult = binomialCoefficient(n, 3);
-        Assert.assertEquals(exactResult, ourResult);
-        Assert.assertEquals(1, ArithmeticUtils.binomialCoefficientDouble(n, 3) / exactResult, 1e-10);
-        Assert.assertEquals(1, ArithmeticUtils.binomialCoefficientLog(n, 3) / FastMath.log(exactResult), 1e-10);
-
-    }
-
-    @Test
-    public void testFactorial() {
-        for (int i = 1; i < 21; i++) {
-            Assert.assertEquals(i + "! ", factorial(i), ArithmeticUtils.factorial(i));
-            Assert.assertEquals(i + "! ", factorial(i), ArithmeticUtils.factorialDouble(i), Double.MIN_VALUE);
-            Assert.assertEquals(i + "! ", FastMath.log(factorial(i)), ArithmeticUtils.factorialLog(i), 10E-12);
-        }
-
-        Assert.assertEquals("0", 1, ArithmeticUtils.factorial(0));
-        Assert.assertEquals("0", 1.0d, ArithmeticUtils.factorialDouble(0), 1E-14);
-        Assert.assertEquals("0", 0.0d, ArithmeticUtils.factorialLog(0), 1E-14);
-    }
-
-    @Test
-    public void testFactorialFail() {
-        try {
-            ArithmeticUtils.factorial(-1);
-            Assert.fail("expecting MathIllegalArgumentException");
-        } catch (MathIllegalArgumentException ex) {
-            // ignored
-        }
-        try {
-            ArithmeticUtils.factorialDouble(-1);
-            Assert.fail("expecting MathIllegalArgumentException");
-        } catch (MathIllegalArgumentException ex) {
-            // ignored
-        }
-        try {
-            ArithmeticUtils.factorialLog(-1);
-            Assert.fail("expecting MathIllegalArgumentException");
-        } catch (MathIllegalArgumentException ex) {
-            // ignored
-        }
-        try {
-            ArithmeticUtils.factorial(21);
-            Assert.fail("expecting MathArithmeticException");
-        } catch (MathArithmeticException ex) {
-            // ignored
-        }
-        Assert.assertTrue("expecting infinite factorial value", Double.isInfinite(ArithmeticUtils.factorialDouble(171)));
+        testAddAndCheckLongFailure(max, max);
+        testAddAndCheckLongFailure(min, min);
     }
 
     @Test
@@ -348,7 +128,7 @@ public class ArithmeticUtilsTest {
         for (int i = 0; i < primeList.length; i++) {
             primes.add(Integer.valueOf(primeList[i]));
         }
-        RandomDataImpl randomData = new RandomDataImpl();
+        RandomDataGenerator randomData = new RandomDataGenerator();
         for (int i = 0; i < 20; i++) {
             Object[] sample = randomData.nextSample(primes, 4);
             int p1 = ((Integer) sample[0]).intValue();
@@ -614,10 +394,10 @@ public class ArithmeticUtilsTest {
             // expected behavior
         }
 
-        Assert.assertEquals(1801088541, ArithmeticUtils.pow(21, 7l));
-        Assert.assertEquals(1, ArithmeticUtils.pow(21, 0l));
+        Assert.assertEquals(1801088541, ArithmeticUtils.pow(21, 7));
+        Assert.assertEquals(1, ArithmeticUtils.pow(21, 0));
         try {
-            ArithmeticUtils.pow(21, -7l);
+            ArithmeticUtils.pow(21, -7);
             Assert.fail("Expecting MathIllegalArgumentException");
         } catch (MathIllegalArgumentException e) {
             // expected behavior
@@ -679,6 +459,102 @@ public class ArithmeticUtilsTest {
 
     }
 
+    @Test(expected=MathArithmeticException.class)
+    public void testPowIntOverflow() {
+        ArithmeticUtils.pow(21, 8);
+    }
+
+    @Test
+    public void testPowInt() {
+        final int base = 21;
+
+        Assert.assertEquals(85766121L,
+                            ArithmeticUtils.pow(base, 6));
+        Assert.assertEquals(1801088541L,
+                            ArithmeticUtils.pow(base, 7));
+    }
+
+    @Test(expected=MathArithmeticException.class)
+    public void testPowNegativeIntOverflow() {
+        ArithmeticUtils.pow(-21, 8);
+    }
+
+    @Test
+    public void testPowNegativeInt() {
+        final int base = -21;
+
+        Assert.assertEquals(85766121,
+                            ArithmeticUtils.pow(base, 6));
+        Assert.assertEquals(-1801088541,
+                            ArithmeticUtils.pow(base, 7));
+    }
+
+    @Test
+    public void testPowMinusOneInt() {
+        final int base = -1;
+        for (int i = 0; i < 100; i++) {
+            final int pow = ArithmeticUtils.pow(base, i);
+            Assert.assertEquals("i: " + i, i % 2 == 0 ? 1 : -1, pow);
+        }
+    }
+
+    @Test
+    public void testPowOneInt() {
+        final int base = 1;
+        for (int i = 0; i < 100; i++) {
+            final int pow = ArithmeticUtils.pow(base, i);
+            Assert.assertEquals("i: " + i, 1, pow);
+        }
+    }
+
+    @Test(expected=MathArithmeticException.class)
+    public void testPowLongOverflow() {
+        ArithmeticUtils.pow(21, 15);
+    }
+
+    @Test
+    public void testPowLong() {
+        final long base = 21;
+
+        Assert.assertEquals(154472377739119461L,
+                            ArithmeticUtils.pow(base, 13));
+        Assert.assertEquals(3243919932521508681L,
+                            ArithmeticUtils.pow(base, 14));
+    }
+
+    @Test(expected=MathArithmeticException.class)
+    public void testPowNegativeLongOverflow() {
+        ArithmeticUtils.pow(-21L, 15);
+    }
+
+    @Test
+    public void testPowNegativeLong() {
+        final long base = -21;
+
+        Assert.assertEquals(-154472377739119461L,
+                            ArithmeticUtils.pow(base, 13));
+        Assert.assertEquals(3243919932521508681L,
+                            ArithmeticUtils.pow(base, 14));
+    }
+
+    @Test
+    public void testPowMinusOneLong() {
+        final long base = -1;
+        for (int i = 0; i < 100; i++) {
+            final long pow = ArithmeticUtils.pow(base, i);
+            Assert.assertEquals("i: " + i, i % 2 == 0 ? 1 : -1, pow);
+        }
+    }
+
+    @Test
+    public void testPowOneLong() {
+        final long base = 1;
+        for (int i = 0; i < 100; i++) {
+            final long pow = ArithmeticUtils.pow(base, i);
+            Assert.assertEquals("i: " + i, 1, pow);
+        }
+    }
+
     @Test
     public void testIsPowerOfTwo() {
         final int n = 1025;
@@ -693,53 +569,6 @@ public class ArithmeticUtilsTest {
         }
     }
 
-    /**
-     * Exact (caching) recursive implementation to test against
-     */
-    private long binomialCoefficient(int n, int k) throws MathArithmeticException {
-        if (binomialCache.size() > n) {
-            Long cachedResult = binomialCache.get(n).get(Integer.valueOf(k));
-            if (cachedResult != null) {
-                return cachedResult.longValue();
-            }
-        }
-        long result = -1;
-        if ((n == k) || (k == 0)) {
-            result = 1;
-        } else if ((k == 1) || (k == n - 1)) {
-            result = n;
-        } else {
-            // Reduce stack depth for larger values of n
-            if (k < n - 100) {
-                binomialCoefficient(n - 100, k);
-            }
-            if (k > 100) {
-                binomialCoefficient(n - 100, k - 100);
-            }
-            result = ArithmeticUtils.addAndCheck(binomialCoefficient(n - 1, k - 1),
-                binomialCoefficient(n - 1, k));
-        }
-        if (result == -1) {
-            throw new MathArithmeticException();
-        }
-        for (int i = binomialCache.size(); i < n + 1; i++) {
-            binomialCache.add(new HashMap<Integer, Long>());
-        }
-        binomialCache.get(n).put(Integer.valueOf(k), Long.valueOf(result));
-        return result;
-    }
-
-    /**
-     * Exact direct multiplication implementation to test against
-     */
-    private long factorial(int n) {
-        long result = 1;
-        for (int i = 2; i <= n; i++) {
-            result *= i;
-        }
-        return result;
-    }
-
     private void testAddAndCheckLongFailure(long a, long b) {
         try {
             ArithmeticUtils.addAndCheck(a, b);
